<?php

namespace App\Http\Controllers;

use App\Services\FirebaseService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Exception;

class OrderController extends Controller
{
    protected $firebaseService;

    public function __construct(FirebaseService $firebaseService)
    {
        $this->firebaseService = $firebaseService;
    }



    public function getOrdersByUser(Request $request)
    {
        try {
            // Allow cross-origin requests
            header("Access-Control-Allow-Origin: *");
            header('Content-type:application/json;charset=utf-8');
            
            // Get the user_id from the request
            $user_id = $request->query('user_id');

            // Validate that the user_id is provided
            if (!$user_id) {
                return response()->json([
                    'status' => false,
                    'message' => 'User ID is required'
                ], 400);
            }

            // SQL query to fetch orders and join order_items based on user_id
            $result = DB::select("
                SELECT orders.*, order_items.*
                FROM orders
                LEFT JOIN order_items ON orders.order_id = order_items.order_id
                WHERE orders.user_id = :user_id
                ORDER BY orders.created_at DESC
            ", ['user_id' => $user_id]);

            // Group the result by order_id
            $orders = [];
            foreach ($result as $row) {
                if (!isset($orders[$row->order_id])) {
                    $orders[$row->order_id] = [
                        'status' => $row->status,
                        'userId' => $row->user_id,
                        'orderNumber' => $row->order_number,
                        'orderId' => $row->order_id,
                        'customerName' => $row->customer_name,
                        'customerAddress' => $row->customer_address,
                        'customerPhoneNumber' => $row->customer_phone_number,
                        'notes' => $row->notes,
                        'deliveryBoyName' => $row->delivery_boy_name,
                        'platform' => $row->platform,
                        'firebaseId' => $row->firebase_id,
                        'firebasePath' => $row->firebase_path,
                        'createdAt' => $row->created_at,
                        'amount' => $row->amount,
                        'discount' => $row->discount,
                        'deliveryCharges' => $row->delivery_charges,
                        'items' => []
                    ];
                }

                $orders[$row->order_id]['items'][] = [
                    'id' => $row->id,
                    'orderId' => $row->order_id,
                    'productName' => $row->product_name,
                    'productId' => $row->product_id,
                    'productPrice' => $row->product_price,
                    'productDiscount' => $row->product_discount,
                    'productNetPrice' => $row->product_net_price,
                    'sku' => $row->sku,
                    'subCategoryId' => $row->sub_category_id,
                    'createdAt' => $row->created_at,
                    'updatedAt' => $row->updated_at,
                    'productImage' => $row->product_image,
                    'quantitySelected' => $row->quantity_selected
                ];
            }

            // Return the orders array in JSON format
            return response()->json(array_values($orders), 200, [], JSON_NUMERIC_CHECK);
            
        } catch (Exception $e) {
            // Handle any exceptions and return an error response
            return response()->json([
                'status' => false,
                'message' => 'An error occurred while fetching the orders.',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function getOrdersReportByDate(Request $request)
    {
        try {
            // Allow cross-origin requests
            header('Access-Control-Allow-Origin: *');
            header('Access-Control-Allow-Methods: GET, POST');
            header('Access-Control-Allow-Headers: Content-Type');
            header('Content-type: application/json;charset=utf-8');
            
            // Get the date range from the request
            $from_date = $request->input('from_date');
            $to_date = $request->input('to_date');

            // Validate the input dates
            if (!$from_date || !$to_date) {
                return response()->json([
                    'status' => false,
                    'message' => 'From date and to date are required'
                ], 400);
            }

            // SQL query to fetch orders and join order_items based on the date range
            $data = DB::select("
                SELECT orders.*, order_items.*
                FROM orders
                JOIN order_items ON orders.order_id = order_items.order_id
                WHERE orders.completed_time >= ? AND orders.completed_time <= ?
                ORDER BY orders.created_at DESC
            ", [$from_date, $to_date]);

            // Initialize variables for tracking totals
            $orderCount = 0;
            $orderAmountTotal = 0;
            $deliveryChargesTotal = 0;
            $orderDiscountTotal = 0;
            $itemDiscountTotal = 0;

            // Initialize an array to hold the unique orders
            $orders = [];

            foreach ($data as $row) {
                $orderId = $row->order_id;

                // Check if order already exists in the orders array
                if (!isset($orders[$orderId])) {
                    // Create order array and populate the properties
                    $order = [
                        'status' => $row->status,
                        'userId' => $row->user_id,
                        'orderNumber' => $row->order_number,
                        'orderId' => $orderId,
                        'customerName' => $row->customer_name,
                        'customerAddress' => $row->customer_address,
                        'customerPhoneNumber' => $row->customer_phone_number,
                        'notes' => $row->notes,
                        'deliveryBoyName' => $row->delivery_boy_name,
                        'platform' => $row->platform,
                        'firebaseId' => $row->firebase_id,
                        'firebasePath' => $row->firebase_path,
                        'createdAt' => $row->created_at,
                        'amount' => $row->amount,
                        'discount' => $row->discount,
                        'deliveryCharges' => $row->delivery_charges,
                        'orderMessage' => $row->order_message,
                        'items' => []
                    ];

                    // Update totals
                    $orderCount++;
                    $orderAmountTotal += $order['amount'];
                    $deliveryChargesTotal += $order['deliveryCharges'];
                    $orderDiscountTotal += $order['discount'];

                    // Add the order to the orders array
                    $orders[$orderId] = $order;
                }

                // Create product array and populate the properties
                $product = [
                    'id' => $row->id,
                    'productName' => $row->product_name,
                    'productId' => $row->product_id,
                    'productPrice' => $row->product_price,
                    'productDiscount' => $row->product_discount,
                    'productNetPrice' => $row->product_net_price,
                    'sku' => $row->sku,
                    'subCategoryId' => $row->sub_category_id,
                    'createdAt' => $row->created_at,
                    'updatedAt' => $row->updated_at,
                    'productImage' => $row->product_image,
                    'quantitySelected' => $row->quantity_selected
                ];

                // Update totals
                $itemDiscountTotal += $product['productDiscount'] * $product['quantitySelected'];

                // Add the product to the existing order's items array
                $orders[$orderId]['items'][] = $product;
            }

            // Create a response object to hold the totals and orders
            $response = [
                'orderCount' => $orderCount,
                'orderAmountTotal' => $orderAmountTotal,
                'deliveryChargesTotal' => $deliveryChargesTotal,
                'orderDiscountTotal' => $orderDiscountTotal,
                'itemDiscountTotal' => $itemDiscountTotal,
                'orders' => array_values($orders)
            ];

            return response()->json($response, 200, [], JSON_NUMERIC_CHECK);
        } catch (Exception $e) {
            // Handle any exceptions and return an error response
            return response()->json([
                'status' => false,
                'message' => 'An error occurred while fetching the orders.',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function saveOrder(Request $request)
    {
        $response = [];
        $data = $request->input('data', []);
        try {
            $data = $request->all();

            $status = $data['status'] ?? null;
            $userId = $data['userId'] ?? null;
            $orderNumber = $data['orderNumber'] ?? null;
            $orderId = $data['orderId'] ?? $data['firebaseId'] ?? null;
            $platform = $data['platform'] ?? null;
            $firebaseId = $data['firebaseId'] ?? null;
            $firebasePath = $data['firebasePath'] ?? null;
            $createdAt = $data['createdAt'] ?? null;
            $created_time = !empty($createdAt) ? date('Y-m-d H:i:s', $createdAt) : null;
            $amount = $data['amount'] ?? null;
            $discount = $data['discount'] ?? null;
            $deliveryCharges = $data['deliveryCharges'] ?? 0;
            $customerName = isset($data['customerName']) ? filter_var($data['customerName'], FILTER_SANITIZE_STRING) : null;
            $customerAddress = isset($data['customerAddress']) ? filter_var($data['customerAddress'], FILTER_SANITIZE_STRING) : null;
            $customerPhoneNumber = isset($data['customerPhoneNumber']) ? filter_var($data['customerPhoneNumber'], FILTER_SANITIZE_STRING) : null;
            $notes = isset($data['notes']) ? filter_var($data['notes'], FILTER_SANITIZE_STRING) : null;
            $deliveryBoyName = isset($data['deliveryBoyName']) ? filter_var($data['deliveryBoyName'], FILTER_SANITIZE_STRING) : null;
            $orderMessage = isset($data['orderMessage']) ? filter_var($data['orderMessage'], FILTER_SANITIZE_STRING) : null;
            $items = isset($data['items']) ? $data['items'] : [];

            // Check if order exists
            $existingOrder = DB::table('orders')->where('order_id', $orderId)->first();
            if ($existingOrder) {
                // Update the order
                DB::table('orders')
                    ->where('order_id', $orderId)
                    ->update([
                        'status' => $status,
                        'user_id' => $userId,
                        'order_number' => $orderNumber,
                        'customer_name' => $customerName,
                        'customer_address' => $customerAddress,
                        'customer_phone_number' => $customerPhoneNumber,
                        'notes' => $notes,
                        'delivery_boy_name' => $deliveryBoyName,
                        'platform' => $platform,
                        'order_message' => $orderMessage,
                        'firebase_id' => $firebaseId,
                        'firebase_path' => $firebasePath,
                        'created_at' => $createdAt,
                        'created_time' => $created_time,
                        'amount' => $amount,
                        'discount' => $discount,
                        'delivery_charges' => $deliveryCharges
                    ]);

                // Delete existing order items
                DB::table('order_items')->where('order_id', $orderId)->delete();

                // Insert new order items
                foreach ($items as $item) {
                    DB::table('order_items')->insert([
                        'order_id' => $orderId,
                        'product_name' => $item['product_name'] ?? null,
                        'product_id' => $item['product_id'] ?? null,
                        'product_price' => $item['product_price'] ?? null,
                        'product_discount' => $item['product_discount'] ?? null,
                        'product_net_price' => $item['product_net_price'] ?? null,
                        'sku' => $item['sku'] ?? null,
                        'sub_category_id' => $item['sub_category_id'] ?? null,
                        'created_at' => $item['created_at'] ?? $createdAt,
                        'created_time' => $item['created_time'] ?? $created_time,
                        'updated_at' => $item['updated_at'] ?? null,
                        'product_image' => $item['product_image'] ?? null,
                        'quantity_selected' => $item['quantity_selected'] ?? null
                    ]);
                }

                $response['message'] = "Order updated successfully";
            } else {
                // Insert new order
                DB::table('orders')->insert([
                    'status' => $status,
                    'user_id' => $userId,
                    'order_number' => $orderNumber,
                    'order_id' => $orderId,
                    'customer_name' => $customerName,
                    'customer_address' => $customerAddress,
                    'customer_phone_number' => $customerPhoneNumber,
                    'notes' => $notes,
                    'delivery_boy_name' => $deliveryBoyName,
                    'platform' => $platform,
                    'order_message' => $orderMessage,
                    'firebase_id' => $firebaseId,
                    'firebase_path' => $firebasePath,
                    'created_at' => $createdAt,
                    'created_time' => $created_time,
                    'amount' => $amount,
                    'discount' => $discount,
                    'delivery_charges' => $deliveryCharges
                ]);

                // Insert new order items
                foreach ($items as $item) {
                    DB::table('order_items')->insert([
                        'order_id' => $orderId,
                        'product_name' => $item['product_name'] ?? null,
                        'product_id' => $item['product_id'] ?? null,
                        'product_price' => $item['product_price'] ?? null,
                        'product_discount' => $item['product_discount'] ?? null,
                        'product_net_price' => $item['product_net_price'] ?? null,
                        'sku' => $item['sku'] ?? null,
                        'sub_category_id' => $item['sub_category_id'] ?? null,
                        'created_at' => $item['created_at'] ?? $createdAt,
                        'created_time' => $item['created_time'] ?? $created_time,
                        'updated_at' => $item['updated_at'] ?? null,
                        'product_image' => $item['product_image'] ?? null,
                        'quantity_selected' => $item['quantity_selected'] ?? null
                    ]);
                }

                // $token = DB::table('fcm_table')
                // ->where('user_id', $userId)
                // ->value('fcm_token');

                // $this->firebaseService->sendNotification($token, "New Order Receive",  $customerName+" place an order", $data);

                $response['message'] = "Order saved successfully";
            }

            return response()->json($response, 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'An error occurred while saving the order.',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    
}
