<?php

namespace App\Http\Controllers;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Exception;

class ProductController extends Controller
{
    public function getProductsBySubCategory(Request $request)
    {
        try {
            // Allow cross-origin requests
            header("Access-Control-Allow-Origin: *");
            header('Content-type:application/json;charset=utf-8');
            
            // Get the sub_category_id from the request
            $subCategoryId = $request->query('id');

            // Validate that the sub_category_id is provided
            if (!$subCategoryId) {
                return response()->json([
                    'status' => false,
                    'message' => 'Sub-category ID is required'
                ], 400);
            }

            // Execute the SQL query to fetch products
            $products = DB::table('products')
                ->where('sub_category_id', $subCategoryId)
                ->where('product_quantity', '>', 0)
                ->get()
                ->toArray();

            // Process the products to check and set 'sku'
            foreach ($products as &$product) {
                if (is_null($product->sku)) {
                    $product->sku = $product->id;
                }
            }

            // Return the data as JSON
            return response()->json($products, 200, [], JSON_NUMERIC_CHECK);
            
        } catch (Exception $e) {
            // Handle any exceptions and return an error response
            return response()->json([
                'status' => false,
                'message' => 'An error occurred while fetching the products.',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function updateProduct(Request $request)
    {
        $response = [];
        
        try {
            $data = $request->all();

            $product_discount = $data['product_discount'] ?? 0;
            $product_price = $data['product_price'] ?? 0;
            $product_image = $data['product_image'] ?? 0;
            $product_net_price = $product_price - $product_discount;
            $product_name = isset($data['product_name']) ? filter_var($data['product_name'], FILTER_SANITIZE_STRING) : null;
            $product_id = $data['product_id'] ?? null;

            // Assuming your database connection is set up properly in the .env file

            // Update the product
            $affected = DB::table('products')
                ->where('product_id', $product_id)
                ->update([
                    'product_name' => $product_name,
                    'product_price' => $product_price,
                    'product_discount' => $product_discount,
                    'product_net_price' => $product_net_price,
                    'product_image' => $product_image
                ]);

            if ($affected) {
                $response['success'] = true;
                $response['message'] = "Product updated successfully.";
            } else {
                $response['success'] = false;
                $response['message'] = "Failed to update product.";
            }

            return response()->json($response, 200);
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'message' => 'An error occurred while updating the product.',
                'error' => $e->getMessage()
            ], 500);
        }
    }


    public function uploadJsonProduct(Request $request)
    {
        ini_set('memory_limit', '256M'); // Increase memory limit if necessary

        try {
            // Check if file exists in request
            if (!$request->hasFile('json_file')) {
                return response()->json(['error' => 'No file uploaded.'], 400);
            }
    
            // Validate the file
            $validator = Validator::make($request->all(), [
                'json_file' => 'required|file|mimetypes:application/json,text/plain',
            ]);
    
            if ($validator->fails()) {
                return response()->json(['error' => $validator->errors()], 400);
            }
    
            // Debugging: Check file details
            $file = $request->file('json_file');
            if ($file === null) {
                return response()->json(['error' => 'File is null.'], 400);
            }
    
            // Debugging: Check file path
            $filePath = $file->getPathname();
            if (!file_exists($filePath)) {
                return response()->json(['error' => 'File does not exist at path: ' . $filePath], 400);
            }
    
            $fileContents = file_get_contents($filePath);
    
            // Debugging: Check file contents
            if ($fileContents === false) {
                return response()->json(['error' => 'Unable to read file contents.'], 400);
            }
    
            $jsonData = json_decode($fileContents, true);
    
            if (json_last_error() !== JSON_ERROR_NONE) {
                return response()->json(['error' => 'Error decoding JSON: ' . json_last_error_msg()], 400);
            }
    
            DB::beginTransaction();
    
            foreach ($jsonData as $item) {
                $price_net = $item['retail'] - $item['com_disc_amt'];
    
                DB::table('products')->updateOrInsert(
                    ['product_id' => $item['item_id']],
                    [
                        'product_name' => $item['item_name'],
                        'product_discount' => $item['com_disc_amt'],
                        'product_price' => $item['retail'],
                        'product_quantity' => ($item['active_ind'] == 'In-Active' || $item['qty'] < 1 || strtotime($item['modified_dte']) < strtotime('-6 months')) ? 0 : $item['qty'],
                        'product_net_price' => $price_net,
                        'active_ind' => $item['active_ind'],
                        'modified_dte' => $item['modified_dte']
                    ]
                );
            }
    
            DB::commit();
            return response()->json(['success' => 'Data imported successfully.'], 200);
        } catch (Exception $e) {
            DB::rollBack();
            // Return detailed error message
            return response()->json(['error' => 'Error: ' . $e->getMessage()], 500);
        }
    }

}
